clear; %clc;  close all;
delete(gcp('nocreate'))
mypool = parpool(4);
format long g
   

%% TARGETS:
n0   = 0.941;           %target employment rate
f0   = 1-(1-0.41)^0.5;	%target biweekly job finding probability (mo. job finding probability is .41)
s0   = f0*(1- n0)/n0;   %implied E-->U transition probability
Theta0 = 0.53;          %target market tightness
q0   = f0/Theta0;   
felas= 0.33;            % = dlog(f)/dlog(theta), elasticity of job finding probability to tightness
mcons= f0*(Theta0^-felas); %scaling parameter in matching function, eg, f = mcons*(Theta^felas);
prNOadj0 = 0.8;         %target adjustment probability



%% CALIBRATION
beta  = 0.96^(1/26);    %biweekly calibration. 0.96 is annual discount factor.
eta   = 0.009;          %worker bargaining power. Targets elasticity of avg. wage w.r.t. employment          
alpha = 0.64;
Alpha = alpha*(1-eta) /(1-eta*(1-alpha));	% Alpha and prodconst for use in static analysis below.
prodconst   = (1-eta) /(1-eta*(1-alpha));   
b     = 0.194;          %flow outside option
training = 7.2;
postingcost = 0.084; 
recruitingcost = postingcost/q0;
HireCost0 = recruitingcost + training;
FireCost0 = 0;
Omega0    = eta*beta*( postingcost*Theta0 + f0*training) + (1-eta)*b; % threat point in bargaining - see below.
pt = 1;                 %aggregate TFP (=1 for steady state analysis)


%% IDIOSYNCRATIC PRODUCTIVITY
%  Set persistence and std dev
rhox       = 0.70^(1/6);    
Stdev_epsx = 0.829/sqrt(1+rhox^2 + rhox^4 + rhox^6 + rhox^8 + rhox^10);	
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );

% LEGENDRE
Fxmax   = 0.999;
Fxmin   = 0.001;
x1      = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN      = exp( E_logx + Stdev_logx*norminv(Fxmax) );
xobs    = 120;
[xgrid, xweights] = legendre(x1, xN, xobs);


%% AGGREGATE PRODUCTIVITY
%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.95^(1/6);            % 0.95 is qtly persistence parameter. 
E_p        = 1;
Stdev_logp = 0.0152;                % targets 'typical' fall in y/n in recession
Var_logp   = Stdev_logp^2;
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 9;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp; 
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2; 
end


%% Employment grid
 % Use static labor demand solution to guide choice of upper bound of grid.
nmin   = 1;
nmax   = 225; % determine based on inspection of steady state density
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% STEADY STATE

% Policy function
[nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf, Xh,Xf] = policyss(beta,alpha,prodconst,  xobs,xgrid,xweights,Stdev_epsx,rhox,mux,   nobs,ngrid,  Omega0,HireCost0,FireCost0, pt);
ssnh = nh;
ssnf = nf;
ssv_no = v_no;
ssv_adjh = v_adjh;
ssv_adjf = v_adjf;
ssvtomaxh= vtomaxh;
ssvtomaxf= vtomaxf;
splhire  = spline(xgrid,nh);
splfire  = spline(xgrid,nf);
% NOTE: Xh, Xf are adjustment Thresholds:
% The firm hires if x >Xh and fires if x <Xf.
%plot(ngrid,Xh,'b-s',   ngrid,Xf,'r-p'); legend('hiring threshold', 'firing threshold'); pause;



% Simulate steady state
rng(502);
N      = 250000;
T      = 50*26;
burnin = 5*26;
        
% Idiosyncratic TFP
xlag  = max(  xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(E_x) + .5*Stdev_epsx*randn(N,1) )));  % scale down so 'nlag' below will be sure to lie within grid.
xfirst= xlag;
xsim  = zeros(N,T);
for t = 1:T
    xt  = max(xgrid(1), min(xgrid(xobs),  exp( mux + rhox*log(xlag) +   Stdev_epsx*randn(N,1) )));
    xsim(:,t) = xt;
    xlag      = xt;
end
    
tt = 0;
nlag  = round( (Alpha*xfirst/Omega0).^(1/(1-alpha)) ); % round up.
nlag6 = nlag;
Nsim  = zeros(T,1);
Seps  = Nsim;
Hires = Nsim;
PrNOadj = zeros(floor(T/6),1);
for t = 1:T
    xt  = xsim(:,t);

     % Interpolate policy rule and round
    nhire  = max(1, round(ppval( splhire, xt)));
    nfire  = max(1, round(ppval( splfire, xt)));
    
     % Optimal labor demand:
    nt    = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
    Nsim(t) = mean(nt);
    Seps(t) = mean( (nlag-nt).*(nt <nlag) );
    Hires(t)= mean( (nt-nlag).*(nt >nlag) );
    
    if t/6 ==floor(t/6)
        tt = tt +1;
        dlogn  = log(nt ./nlag6);
        PrNOadj(tt) = mean((nt ==nlag6));
        nlag6 = nt;
    end 
    nlag = nt;
    if t/52==floor(t/52); disp( [t,mean(Nsim(52:t))] ); end
end
clear nt xt;
N0 = mean(Nsim(burnin+1:end));
Seps0 = mean(Seps(burnin+1:end));
Hires0  = mean(Hires(burnin+1:end));
PrNOadj0 = mean( PrNOadj(ceil(burnin/6)+1:end) );
 %Labor force consistent with stipulated theta, @ target N/L:
L0 = N0 + (Hires0/f0);
disp('STEADY STATE: Labor Force, Employment, Employment rate, and Prob of not adjusting');
disp( [L0, N0, N0/L0, PrNOadj0] );
xss = xsim(:,T);
nss = nlag;
%
% STEADY STATE: Labor Force, Employment, Employment rate, and Prob of not adjusting
%           19.9704163240147          18.7651843008547         0.939649128811067         0.798456226804123