clear;
delete(gcp('nocreate'))
mypool = parpool(4);
format long g


%% TARGETS:
n0   = 0.941;           %target employment rate
f0   = 1-(1-0.41)^0.5;	%target biweekly job finding probability (mo. job finding probability is .41)
s0   = f0*(1- n0)/n0;   %implied E-->U transition probability
Theta0 = 0.53;          %target market tightness
q0   = f0/Theta0;   
felas= 0.33;            % = dlog(f)/dlog(theta), elasticity of job finding probability to tightness
mcons= f0*(Theta0^-felas); %scaling parameter in matching function, eg, f = mcons*(Theta^felas);
prNOadj0 = 0.667;       %target adjustment probability



%% CALIBRATION
beta  = 0.96^(1/26);    %biweekly calibration. 0.96 is annual discount factor.
eta   = 0.026;          %worker bargaining power. Targets elasticity of avg. wage w.r.t. employment          
alpha = 0.64;
Alpha = alpha*(1-eta) /(1-eta*(1-alpha));	% Alpha and prodconst for use in static analysis below.
prodconst   = (1-eta) /(1-eta*(1-alpha));   
b     = 0.21;           %flow outside option
training = 2.42;
postingcost = 0.09; 
recruitingcost = postingcost/q0;
HireCost0 = recruitingcost + training;
FireCost0 = 0;
Omega0    = eta*beta*( postingcost*Theta0 + f0*training) + (1-eta)*b; % threat point in bargaining - see below.
pt = 1;                 %aggregate TFP (=1 for steady state analysis)


%% IDIOSYNCRATIC PRODUCTIVITY
%  Set persistence and std dev
rhox       = 0.70^(1/6);    
Stdev_epsx = 0.471/sqrt(1+rhox^2 + rhox^4 + rhox^6 + rhox^8 + rhox^10);	
%Unconditional moments:
E_x        = 1;
Stdev_logx = Stdev_epsx/sqrt(1-rhox^2);
Var_logx   = Stdev_logx^2;
E_logx     = log(E_x) - 0.5*Var_logx;
mux        = (1-rhox)*E_logx;
Stdev_x    = sqrt( (exp(Var_logx)-1)*exp(2*E_logx + Var_logx) );

% LEGENDRE
Fxmax   = 0.999;
Fxmin   = 0.001;
x1      = exp( E_logx + Stdev_logx*norminv(Fxmin) );
xN      = exp( E_logx + Stdev_logx*norminv(Fxmax) );
xobs    = 150;
[xgrid, xweights] = legendre(x1, xN, xobs);
 % xobs and xobs2 set higher than in Krusell-Smith in order 
 % to refine calculation of flow balance. KS coefficients
 % not really sensitive to these gridding differences.



%% AGGREGATE PRODUCTIVITY
%%%%%%%%%%%%%%%%%%%%%%%%%
rhop       = 0.95^(1/6);            % 0.95 is qtly persistence parameter. 
E_p        = 1;
Stdev_logp = 0.0172;                % targets 'typical' fall in y/n in recession
Var_logp   = Stdev_logp^2;
Stdev_epsp = Stdev_logp*sqrt(1-rhop^2);
E_logp     = log(E_p) - 0.5*Var_logp;
mup        = (1-rhop)*E_logp;
Stdev_p    = sqrt( (exp(Var_logp)-1)*exp(2*E_logp + Var_logp) );
Var_epsp   = Stdev_epsp^2;

% Productivity grid: Rouwenhorst transition matrix
pobs   = 9;
logp0  = -sqrt(pobs-1)*Stdev_logp;
logpT  =  sqrt(pobs-1)*Stdev_logp;
logpgrid_ = (logp0 : (logpT-logp0)/(pobs-1) : logpT)';
logpgrid  = logpgrid_+ E_logp; 
pgrid  = exp(logpgrid);
prob1  = (1+rhop)/2;
prob2  = (1+rhop)/2;
Ptrans = [ prob1, 1-prob1; 1-prob2, prob2];
for j  = 1:pobs-2
    Np = j+2;
    Ptrans = prob1*[Ptrans, zeros(Np-1,1); zeros(1,Np-1), 0]  + ...
            (1-prob1)* [zeros(Np-1,1) Ptrans; 0 zeros(1,Np-1)] + ...
            (1-prob2)* [zeros(1,Np-1) 0; Ptrans zeros(Np-1,1)] + ...
            prob2*[0 zeros(1,Np-1); zeros(Np-1,1) Ptrans];
    Ptrans(2:end-1,:) = Ptrans(2:end-1,:)/2; 
end



%% Employment grid
 % Use static labor demand solution to guide choice of upper bound of grid.
nmin   = 1;
nmax   = 175; 
ngrid  = (nmin :1: nmax)';
nobs   = length(ngrid);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% STEADY STATE

% Policy function
[nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf, Xh,Xf] = policyss(beta,alpha,prodconst,  xobs,xgrid,xweights,Stdev_epsx,rhox,mux,   nobs,ngrid,  Omega0,HireCost0,FireCost0, pt);
ssnh = nh;
ssnf = nf;
ssv_no = v_no;
ssv_adjh = v_adjh;
ssv_adjf = v_adjf;
ssvtomaxh= vtomaxh;
ssvtomaxf= vtomaxf;
% NOTE: Xh, Xf are adjustment Thresholds:
% The firm hires if x >Xh and fires if x <Xf.
%plot(ngrid,Xh,'b-s',   ngrid,Xf,'r-p'); legend('hiring threshold', 'firing threshold'); pause;


% Simulate steady state
% 1] Expand productivity grid %%%%%%%%%%%%%%%%%%%%%%
%    First step, make EQUIPROBABLE grid:
pstep = 0.0025;
pindx = (0 :pstep: 1)';
pindx(end) = 0.999;
pindx(1) = 0.001;
xgrid2 = exp( E_logx + Stdev_logx*norminv(pindx));

%     Second step: Equiprobable leaves large gaps  
%     between xs at high x. Fill in these gaps.
dtol  = 0.005;	% N.B. choose largest gap between xs
diffx = xgrid2(2:end) - xgrid2(1:end-1);
pos   = find(diffx <=dtol,1,'last');    % N.B. This presumes difference of xs >dtol ONLY at high x.
addx  = ( xgrid2(pos):  dtol   :xgrid2(end))';
addx  = unique( [addx; xgrid2(end)] );
xgrid2  = [xgrid2(1:pos-1); ...
           addx];
xobs2   = length(xgrid2);
disp('No. of productivity obs. for use in sims'); disp(xobs2);


% 2]  Transition matrix %%%%%%%%%%%%%%%%%%%%%%
xlo = xgrid2(1);
xhi = xgrid2(end);
transxmat2 = zeros(xobs2,xobs2);
parfor i = 1:xobs2
    xlag = xgrid2(i);
    F1   = normcdf( (log(xlo)     -mux -rhox*log(xlag)) /Stdev_epsx);
    FN   = normcdf( (log(xhi)     -mux -rhox*log(xlag)) /Stdev_epsx);
    
    for j = 1:xobs2
        if 2 <=j && j <=xobs2 -1
            boundl = xgrid2(j) -0.5*(xgrid2(j  ) -xgrid2(j-1));
            boundr = xgrid2(j) +0.5*(xgrid2(j+1) -xgrid2(j  ));
            transxmat2(i,j) = normcdf( (log(boundr) -mux -rhox*log(xlag)) /Stdev_epsx) - ...
                              normcdf( (log(boundl) -mux -rhox*log(xlag)) /Stdev_epsx);
        elseif j ==1
            boundr = xgrid2(j) +0.5*(xgrid2(j+1) -xgrid2(j  ));
            transxmat2(i,j) = normcdf( (log(boundr) -mux -rhox*log(xlag)) /Stdev_epsx) -F1;
            
        elseif j ==xobs2
            boundl = xgrid2(j) -0.5*(xgrid2(j  ) -xgrid2(j-1));
            transxmat2(i,j) = FN - normcdf( (log(boundl) -mux -rhox*log(xlag))/Stdev_epsx);
        end
        transxmat2(i,j) = transxmat2(i,j) /(FN-F1);
    end
    
end
%transxmat2=transxmat2./(sum(transxmat2,2)*ones(1,xobs2));


% 3] Interpolate value function to new grid %%%%%%%%%%%%%%%%%%%%%%
vhfit = spline(xgrid',ssvtomaxh',xgrid2);
vffit = spline(xgrid',ssvtomaxf',xgrid2);
[~,posh] =  max(vhfit);
[~,posf] =  max(vffit);
nh2   = ngrid(posh');
nf2   = ngrid(posf');
[~,hro] = min(abs( ngrid*ones(1,xobs2) -ones(nobs,1)*nh2')); %this finds position in ngrid that matches "nh2"
[~,fro] = min(abs( ngrid*ones(1,xobs2) -ones(nobs,1)*nf2'));
hro = hro';
fro = fro';


% 4] Initialize joint distribution of (n,x).  %%%%%%%%%%%%%%%%%%%%%%
% This uses 'brute-force' simulations to develop
% an initial guess for the joint distribution. 
% Later, we use Eric Young's nonstochastic
% simulator to refine.
N = 250000;
T = 26*5;
 % Draw productivity:
xsim = zeros(N,T);
xlag = E_x*ones(N,1);
for t = 1:T
    u    = rand(N,1);
    F1   = normcdf( (log(xgrid(1   )) -mux -rhox*log(xlag))/Stdev_epsx);
    FN   = normcdf( (log(xgrid(xobs)) -mux -rhox*log(xlag))/Stdev_epsx);
    xsim(:,t) = exp( mux + rhox*log(xlag) + Stdev_epsx*norminv(u.*(FN-F1) +F1));
    xlag      = xsim(:,t);
end
nlag  = round( (Alpha*xlag/Omega0).^(1/(1-alpha)) );

 % Now, simulate labor demand choices.
splhire  = spline(xgrid,nh);
splfire  = spline(xgrid,nf);
Nsim  = zeros(T,1);
for t = 1:T
    xt  = xsim(:,t);

     % Interpolate policy rule and round
    nhire  = max(1, round(ppval( splhire, xt)));
    nfire  = max(1, round(ppval( splfire, xt)));
    
     % Optimal labor demand:
    nt    = nlag.*(nhire<=nlag).*(nlag<=nfire) + nhire.*(nlag<nhire) + nfire.*(nlag>nfire);
    Nsim(t) = mean(nt);

    nlag = nt;    
end


% 5] Solve for steady state joint distribution of (n,x).  %%%%%%%%%%%%%%%%%%%%%%
 % First, get joint distribution implied by the sims. This is h0.
 % Then, from h0, use Young's nonstochastic simulator to 
 % solve for steady state h.
h_xn  = zeros(xobs2,nobs);
parfor i = 1:nobs
    npos = find(nt ==ngrid(i));
    if isempty(npos) ==0
        for j = 1:xobs2
            if 2<=j && j<=xobs2-1
                boundl = xgrid2(j) -0.5*(xgrid2(j  ) -xgrid2(j-1));
                boundr = xgrid2(j) +0.5*(xgrid2(j+1) -xgrid2(j  ));
                xpos = find(boundl<= xt(npos) & xt(npos) <=boundr);
                h_xn(j,i) = length(xpos) /N; % share of simulated firms with nt=ngrid(i) 
                                             % x 'near' xgrid(j).
            elseif j ==1
                boundr = xgrid2(j) +0.5*(xgrid2(j+1) -xgrid2(j  ));
                xpos = find(xt(npos) <=boundr);
                h_xn(j,i) = length(xpos) /N;
            elseif j ==xobs2
                boundl = xgrid2(j) -0.5*(xgrid2(j  ) -xgrid2(j-1));
                xpos = find(boundl<= xt(npos));
                h_xn(j,i) = length(xpos) /N;
            end 
        end
    end
end
h0_xn= h_xn;

 % Beginning from h0, simulate forward:
T   = 26*25;
Nsim  = zeros(T,1);
Seps  = Nsim;
Hires = Nsim;
for t = 1:T
    
    [Nt,St,Ht, hnew_xn, ~] = jointxn(xobs2,xgrid2,nobs,ngrid,  transxmat2,  h_xn,  Xh,Xf, nh2,nf2, hro,fro);
    
    Nsim(t)  = Nt;
    Seps(t)  = St;
    Hires(t) = Ht;
    
    h_xn = hnew_xn;
    %if t/50 ==floor(t/50); disp(t); end
end

burnin = 26*5;
N0 = mean(Nsim(burnin+1:end));
Seps0 = mean(Seps(burnin+1:end));
Hires0  = mean(Hires(burnin+1:end));
 %Labor force consistent with stipulated theta, @ target N/L:
L0 = N0 + (Hires0/f0);
disp('STEADY STATE: Labor Force, Employment, Employment rate');
disp( [L0, N0, N0/L0] );

hn = sum(hnew_xn);
hss_xn = h_xn;
Nss= N0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% IRFs
 
 % (I)  AGGREGATE LABOR DEMAND
Nl   = N0*0.94;
Nr   = N0*1.06;

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % Grids are structured so more points are near N0, the
 % steady state. Going up from N0, the step size 
 % increases by s%, such that 
 %
 % Ngrid(i+1)= Ngrid(i) + step(i+1); and
 % step(i+1) = step(i)*(1+s).
 %
 % Likewise, going down from N0, the step size falls by s%.
 % Given an initial step size and the terminal point 
 % Nr (or, Nl), the solution for s is given 
 % implicitly by 
 %
 % Nr = N0 + initial step* {sum of(1+s)^i from i=0 to i=r-1} 
 %
 % where r is # of grid points to right of N0. The choice 
 % of r is based on inspection, namely, an r that implies
 % a tolerably sized terminal step. 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
  % A) Make UPPER half of grid from N0 to Nr==max{N}
step1 = 0.15;	% initial step
obs   = 5;      % # of grid points to right of N0.       
tindx = (0:1:obs-1)';
% Solve for unknown 's'.
smin =  0;	% Lower support.
smax = .5;	% Plausible max on s.
s    = .5*(smin + smax);
Nerr =  1;
while Nerr >0.001
    lhs  = (Nr-N0)/step1;
    rhs  = sum( (1+s).^tindx);

    nerr = lhs - rhs;
    if nerr >0
        smin = s;
    elseif nerr <0
        smax = s;
    end
    s    = .5*(smin + smax);
    
    Nerr = abs(nerr);
   %disp( [nerr, smin, smax] ); pause;
end
% Grid to right of N0:
Nrgrid= zeros(obs,1);
step  = step1;
Nlast = N0;
for i = 1:obs
    Nrgrid(i) = Nlast + step;
    step  = step*(1+s);
    Nlast = Nrgrid(i);
end

  % B) Make LOWER half of grid from N0 to Nl==min{N}
step1= 0.15;	% initial step
obs   = 5;      % # of grid points to left of N0.       
tindx = (0:1:obs-1)';
% Solve for unknown 's'.
smin =  0;	% Lower support.
smax = .5;	% Plausible max on s.
s    = .5*(smin + smax);
Nerr =  1;
while Nerr >0.001
    lhs  = (N0-Nl)/step1;
    rhs  = sum( (1+s).^tindx);

    nerr = lhs - rhs;
    if nerr >0
        smin = s;
    elseif nerr <0
        smax = s;
    end
    s    = .5*(smin + smax);
    
    Nerr = abs(nerr);
   %disp( [nerr, smin, smax] ); pause;
end
% Grid to left of N0:
Nlgrid= zeros(obs,1);
step  = step1;
Nlast = N0;
for i = 1:obs
    Nlgrid(i) = Nlast - step;
    step  = step*(1+s);
    Nlast = Nlgrid(i);
end

 % C) Piece grids together
Nvec = sort( [Nlgrid; N0; Nrgrid] );
disp('Employment vector');
disp(Nvec);
Nobs = length(Nvec);
 

 % (II)  AGGREGATE MARKET TIGHTNESS
 % NOTE: q = mcons*(theta^(felas-1)).
 %       Since max of q is 1, min of theta is (1/mcons)^(1/(felas-1)). 
Thl  = 0.01 + (1/mcons)^(1/(felas-1)); 
Thr  = 2.8*Theta0; 

 % Tightness grid is also pieced together from 2 sub-grids. As  
 % for how points are spaced, there are 2 competing considerations. 
 % First, following how Ngrid was constructed, we want to mass points
 % near the steady state. Second, q(Theta) [the job filling rate]
 % is highly nonlinear and falls rapidly near low Theta. 
 % This suggests massing points near low Theta. 
 % We try to do both: (i) for subgrid >Theta0, points are 
 % concentrated near Theta0; (ii) for subgrid <Theta0,
 % points are concentrated near small Theta0.
 
 % First, upper half of grid
step1 = 0.045;
obs   = 11;
tindx = (0:1:obs-1)';
smin =  0;  
smax = .5;	
s    = .5*(smin + smax);
Terr =  1;
while Terr >0.001
    lhs  = (Thr-Theta0)/step1;
    rhs  = sum( (1+s).^tindx);

    terr = lhs - rhs;
    if terr >0
        smin = s;
    elseif terr <0
        smax = s;
    end
    s    = .5*(smin + smax);
    
    Terr = abs(terr);
   %disp( [terr, smin, smax] ); pause;
end
% Grid to right of Theta0:
Thrgrid= zeros(obs,1);
step   = step1;
Thlast = Theta0;
for i = 1:obs
    Thrgrid(i) = Thlast + step;
    step   = step*(1+s);
    Thlast = Thrgrid(i);
end

 % Second, lower half of grid.
step1 = 0.045;
obs   = 11;	
tindx = (0:1:obs-1)';
smin =  0;  
smax = .5;	
s    = .5*(smin + smax);
Terr =  1;
while Terr >0.001
    lhs  = (Theta0-Thl)/step1;
    rhs  = sum( (1-s).^tindx); % NOTE: 1 MINUS s, since the step size will FALL as theta descends.

    terr = lhs - rhs;
    if terr >0
        smax = s;
    elseif terr <0
        smin = s;
    end
    s    = .5*(smin + smax);
    
    Terr = abs(terr);
   %disp( [terr, smin, smax] ); pause;
end
% Grid to left of Theta0:
Thlgrid= zeros(obs,1);
step   = step1;
Thlast = Theta0;
for i = 1:obs
    Thlgrid(i) = Thlast - step;
    step   = step*(1-s);
    Thlast = Thlgrid(i);
end    

 % Now piece together:
Thvec = sort( [Thlgrid; Theta0; Thrgrid] );
Thobs = length(Thvec);

 % Implied hiring cost
Fvec  = mcons*(Thvec.^felas);
Qvec  = mcons*(Thvec.^(felas-1));
Omega = eta*beta*( postingcost*Thvec + training*Fvec) + (1-eta)*b;
HireCost = (postingcost./Qvec) + training;
disp('Market tightness, Finding rate, Filling rate');
disp( [Thvec, Fvec, Qvec] );



 % (III) KS COEFFICIENTS
NOLSp    = 0.0737266150442348;
NOLSlag  = 0.857410968556065;
NOLScons = 0.416993369770869;
ThOLSp   = 12.4939302938629;
ThOLSn   = 23.246798924626;
ThOLScons= -68.6391439172459;
KScoefN  = [NOLScons;  NOLSp;  NOLSlag];
KScoefT  = [ThOLScons; ThOLSp; ThOLSn];
KScoef   = [ KScoefN; KScoefT ];


 % (IV) AGGREGATE TFP
T    = 6 * 13;    % 6 fortnights per qt
pirf = zeros(T,1);
epsp = log(1.01) -( E_logp + 0.5*Var_logp  - 0.5*Var_epsp );
for t= 1:T;  pirf(t)= exp(E_logp + 0.5*Var_logp) * exp( (rhop^(t-1))*epsp  - 0.5*(rhop^(2*(t-1)))*Var_epsp ); end
burnin = 2*T; 
pirf = [ones(burnin,1); pirf];  % Fix at 1 for 2 years, let it stabilize.
T = length(pirf);



%% Policy Function
[nh,nf, v_no,v_adjh,v_adjf, vtomaxh,vtomaxf,  Xhmat,Xfmat] = policysearch(alpha,beta,   pobs,pgrid,Ptrans,  xobs,xgrid,xweights,mux,rhox,Stdev_epsx,   nobs,ngrid,   eta,Omega,prodconst,HireCost, Nvec,Nobs,NOLSp,NOLSlag,NOLScons, Thvec,Thobs,ThOLSp,ThOLSn,ThOLScons);
nhmat = reshape(nh,xobs,pobs,Thobs,Nobs);
nfmat = reshape(nf,xobs,pobs,Thobs,Nobs);
vhmat = reshape( vtomaxh, xobs,pobs,Thobs,Nobs,nobs);
vfmat = reshape( vtomaxf, xobs,pobs,Thobs,Nobs,nobs);
 % For simulation, helpful to have threshold functions
 % w.r.t. x - threshold Xhmat (Xfmat) such that firm 
 % hires (fires) if x >Xhmat (x <Xfmat).
     
    

%% Simulate
tol=1e-12;
Nlag  = N0;
Thlag = Theta0;
hlag_xn = hss_xn;
hnlag = sum(hlag_xn);
Nsim  = zeros( T,1);
Thsim = zeros( T,1);
SSNsim= zeros( T,1);
for t = 1:T

    %% Draw agg productivity
    pt   = pirf(t); 
    pos2 = find( pgrid >pt,1,'first');
    pos1 = pos2 -1;
    plo  = pgrid( pos1);
    phi  = pgrid( pos2);
        
    %% Interpolate value functions and thresholds at p(t)
    vh_p1= reshape(vhmat(:,pos1,:,:,:),  xobs,Thobs,Nobs,nobs);
    vh_p2= reshape(vhmat(:,pos2,:,:,:),  xobs,Thobs,Nobs,nobs);
    vf_p1= reshape(vfmat(:,pos1,:,:,:),  xobs,Thobs,Nobs,nobs);
    vf_p2= reshape(vfmat(:,pos2,:,:,:),  xobs,Thobs,Nobs,nobs);
    vhfitp = vh_p1 + ((pt-plo)/(phi-plo+tol))*( vh_p2 -vh_p1);
    vffitp = vf_p1 + ((pt-plo)/(phi-plo+tol))*( vf_p2 -vf_p1);

    Xh_p1= reshape(Xhmat(:,pos1,:,:),  nobs,Thobs,Nobs);
    Xh_p2= reshape(Xhmat(:,pos2,:,:),  nobs,Thobs,Nobs);
    Xf_p1= reshape(Xfmat(:,pos1,:,:),  nobs,Thobs,Nobs);
    Xf_p2= reshape(Xfmat(:,pos2,:,:),  nobs,Thobs,Nobs); 
    Xhfitp = Xh_p1 + ((pt-plo)/(phi-plo+tol))*( Xh_p2 -Xh_p1);
    Xffitp = Xf_p1 + ((pt-plo)/(phi-plo+tol))*( Xf_p2 -Xf_p1);


    %% Solve Tightness and Employment. 
     % A nested fixed point problem:
     % * Conjecture N(t), Theta(t). 
     % * Solve inner fixed point problem for N(t) given conjecture for Theta(t). 
     % * Compute implied Theta(t) and check against initial conjecture.
     %   Update conjecture for Theta(t) and repeat.
    Ntin = Nlag;
    Thetatin = Thlag;
    [Thetat, Nt, ht_xn] = Thetaouterloop(Ntin, Thetatin, Thvec,Thobs,Nvec,Nobs,L0,  vhfitp,vffitp,Xhfitp,Xffitp,  xobs2,xobs,nobs, xgrid2,xgrid,ngrid,  transxmat2,hlag_xn,  mcons,felas);
    Nsim(t) = Nt;
    Thsim(t)= Thetat;

     % Flow Balance:
    outflow = flowbalance(Nt, Thetat, Thvec,Thobs,Nvec,Nobs,L0,  vhfitp,vffitp,Xhfitp,Xffitp,  xobs2,xobs,nobs, xgrid2,xgrid,ngrid,  transxmat2,hlag_xn,  mcons,felas);
    hn = sum(ht_xn);
    dh = hn - hnlag;
     % I)  Bin (very thin) right tail for results more robust to gridding, size of impulse, etc.
    Hn = cumsum(hn);
    posup = find(Hn < .999, 1, 'last');
    hnup  = sum( hn(posup+1:end) );
    hnlagup = sum( hnlag(posup+1:end) );
    outflowup = outflow(posup+1:end)*(hnlag(posup+1:end)'/hnlagup);
    dhup = hnup - hnlagup;
    dh_  = [dh(1:posup), dhup];
    hnlag_ = [hnlag(1:posup), hnlagup];
    outflow_ = [outflow(1:posup), outflowup];
    hss_   = (dh_./(tol+outflow_)) + hnlag_;
    poslast= find(hn >0, 1, 'last');
    nup    = median( ngrid(posup+1: poslast) );
    ngrid_ = [ngrid(1:posup);  nup];
     % II)  Flow balance==0 if outflow = 0 (which means, almost surely, that h_-1=0) 
    hss_   = hss_ .*( outflow_ >0 );
     % III) Normalize
    hss_   = hss_ / sum(hss_); 
     % IV)  Calculate:
    SSNsim(t) = hss_ * ngrid_;

     
     % reset:
    hnlag  = hn;
    Thlag  = Thsim(t);
    Nlag   = Nsim(t);
    hlag_xn= ht_xn;
end

% Frictionless
En = 20;
Ex_a   = exp((1/(1-alpha))*E_logx  + 0.5*(((1/(1-alpha))*Stdev_logx)^2));
Ex_a   = Ex_a * ((normcdf((log(xgrid(end))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx) - ...
                  normcdf((log(xgrid(1  ))-E_logx-(1/(1-alpha))*Var_logx)/Stdev_logx)) / (Fxmax-Fxmin));
frisch = 0.5;
W      = alpha * ((Ex_a/En)^(1-alpha));
Nstar  = (Ex_a^((1-alpha)/(1+(1/frisch)-alpha))) * ( (alpha*pirf*(Nss^(1/frisch))/W).^(1/(1+(1/frisch)-alpha)) );



% TAKE QTLY AVG.
qts = T/6;
qtroperate = zeros(qts,T);
for i = 1:qts
    cos = (1+(i-1)*6: i*6);
    qtroperate(i,cos) =  1/6;
end
QtNsim   = qtroperate * Nsim;
QtThsim  = qtroperate * Thsim;
QtSSNsim = qtroperate * SSNsim;
QtNstar  = qtroperate * Nstar;

ro1 =  (qts/3) +1;
ro2 = 2*qts/3;
EQtNsim   = mean(QtNsim(  ro1 : ro2));
EQtThsim  = mean(QtThsim( ro1 : ro2));
EQtSSNsim = mean(QtSSNsim(ro1 : ro2));
EQtNstar  = mean(QtNstar( ro1 : ro2));

disp( 'Actual Employment, Flow Balance, and Frictionles' );
disp( [log(QtNsim(ro2+1:end)/EQtNsim),   log(QtSSNsim(ro2+1:end)/EQtSSNsim),   log(QtNstar(ro2+1:end)/EQtNstar)] ); 
